/***********************************************************************************
* Fancy Tasks: Plasmoid providing a fancy representation of your tasks and launchers.
* Copyright (C) 2009 Michal Dutkiewicz aka Emdek <emdeck@gmail.com>
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*
***********************************************************************************/

#ifndef FANCYTASKSICON_HEADER
#define FANCYTASKSICON_HEADER

#include "FancyTasksApplet.h"

#include <QTimer>
#include <QPixmap>
#include <QPainter>
#include <QPointer>
#include <QGraphicsWidget>

#include <KUrl>
#include <KMimeType>
#include <KPropertiesDialog>

#include <taskmanager/task.h>
#include <taskmanager/taskitem.h>
#include <taskmanager/taskactions.h>
#include <taskmanager/taskmanager.h>
#include <taskmanager/groupmanager.h>
#include <taskmanager/abstractgroupingstrategy.h>

#include "ui_group.h"

class FancyTasksApplet;
class FancyTasksLauncher;
class FancyTasksJob;
class FancyTasksToolTip;

using TaskManager::GroupPtr;
using TaskManager::TaskPtr;
using TaskManager::StartupPtr;
using TaskManager::AbstractGroupableItem;
using TaskManager::GroupManager;
using TaskManager::TaskItem;
using TaskManager::TaskGroup;

class FancyTasksIcon : public QGraphicsWidget
{
    Q_OBJECT

    public:
        FancyTasksIcon(TaskManager::AbstractGroupableItem *abstractItem, FancyTasksLauncher *launcher, FancyTasksJob *job, FancyTasksApplet *parent);

        enum ItemType { Other = 0, Launcher, Job, Startup, Task, Group };

        ItemType itemType() const;
        QPointer<FancyTasksLauncher> launcher();
        QList<QPointer<FancyTasksJob> > jobs();
        AbstractGroupableItem* abstractItem();
        TaskGroup* group();
        QString title() const;
        QPainterPath shape() const;
        QRect iconGeometry() const;
        QPixmap iconPixmap(qreal size);
        KIcon icon();
        qreal iconSize() const;
        qreal factor() const;
        bool isVisible() const;
        bool demandsAttention() const;

    protected:
        void paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget);
        void hoverEnterEvent(QGraphicsSceneHoverEvent *event);
        void hoverMoveEvent(QGraphicsSceneHoverEvent *event);
        void hoverLeaveEvent(QGraphicsSceneHoverEvent *event);
        void dragEnterEvent(QGraphicsSceneDragDropEvent *event);
        void dragMoveEvent(QGraphicsSceneDragDropEvent *event);
        void dragLeaveEvent(QGraphicsSceneDragDropEvent *event);
        void dropEvent(QGraphicsSceneDragDropEvent *event);
        void mouseMoveEvent(QGraphicsSceneMouseEvent *event);
        void mousePressEvent(QGraphicsSceneMouseEvent *event);
        void mouseReleaseEvent(QGraphicsSceneMouseEvent *event);
        void contextMenuEvent(QGraphicsSceneContextMenuEvent *event);

    public slots:
        void setFactor(qreal factor);
        void setSize(qreal size);
        void setColor(QColor color);
        void refreshPixmap(bool force = true);
        void startAnimation(int duration, QString animation);
        void stopAnimation();
        void repeatAnimation();
        void zoomAnimation(qreal progress);
        void jumpAnimation(qreal progress);
        void rotateAnimation(qreal progress);
        void bounceAnimation(qreal progress);
        void blinkAnimation(qreal progress);
        void glowAnimation(qreal progress);
        void spotlightAnimation(qreal progress);
        void show();
        void hide();
        void updateSize();
        void taskChanged(::TaskManager::TaskChanges changes);
        void launcherChanged();
        void jobChanged();
        void jobNeedsAttention();
        void gotTaskPointer();
        void setIcon();
        void setThumbnail();
        void setTask(TaskManager::AbstractGroupableItem *abstractItem);
        void setLauncher(FancyTasksLauncher *launcher);
        void addJob(FancyTasksJob *job);
        void removeJob(FancyTasksJob *job);
        void activate();
        void activateWindow(bool force = false);
        void highlightWindow();
        void publishIconGeometry();
        void updateToolTip();
        void updateToolTipPosition();
        void toggleToolTip();
        void blockToolTip();
        void showGroupDialog();
        void groupDialogAccepted();
        void groupItemAdded(AbstractGroupableItem* item);
        void groupItemRemoved(AbstractGroupableItem* item);

    private:
        FancyTasksApplet *m_applet;
        QPointer<FancyTasksToolTip> m_toolTip;
        QPointer<FancyTasksLauncher> m_launcher;
        QList<QPointer<FancyTasksJob> > m_jobs;
        ItemType m_itemType;
        TaskManager::AbstractGroupableItem *m_abstractItem;
        TaskManager::TaskItem *m_task;
        TaskManager::TaskGroup *m_group;
        KDialog *m_groupDialog;
        KIcon m_icon;
        QTimer *m_toolTipTimer;
        QTimer *m_highlightTimer;
        QTimer *m_dragTimer;
        QPixmap m_thumbnail;
        QTimer *m_animationRepeater;
        QPixmap m_pixmap;
        QPixmap m_modifiedPixmap;
        QPixmap m_taskMarkPixmap;
        QPixmap m_spotlightPixmap;
        QString m_title;
        QString m_description;
        QColor m_color;
        WId m_activeWindow;
        FancyTasksApplet::AnimationType m_currentAnimation;
        qreal m_size;
        qreal m_factor;
        qreal m_animationProgress;
        int m_jobsProgress;
        int m_currentAnimationDuration;
        int m_animation;
        int m_itemsAmount;
        bool m_pixmapValid;
        bool m_visible;
        bool m_contextMenuVisible;
        bool m_demandsAttention;
        bool m_toolTipValid;
        bool m_pressed;
        bool m_jobsRunning;
        Ui::group m_groupUi;

    signals:
        void hoverMoved(QObject *item, qreal across);
        void hoverLeft();
};

#endif
