/***********************************************************************************
* Fancy Tasks: Plasmoid providing a fancy representation of your tasks and launchers.
* Copyright (C) 2009 Michal Dutkiewicz aka Emdek <emdeck@gmail.com>
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*
***********************************************************************************/

#ifndef FANCYTASKSLAUNCHER_HEADER
#define FANCYTASKSLAUNCHER_HEADER

#include <QPixmap>
#include <QGraphicsSceneDragDropEvent>

#include <KUrl>
#include <KMenu>
#include <KIcon>
#include <KFileItem>
#include <KDesktopFile>

#include <KIO/CopyJob>
#include <KIO/NetAccess>
#include <KIO/PreviewJob>

class FancyTasksApplet;
class FancyTasksIcon;

class FancyTasksLauncher : public QObject
{
    Q_OBJECT

    public:
        FancyTasksLauncher(KUrl url, FancyTasksApplet *applet);
        ~FancyTasksLauncher();

        void dropEvent(FancyTasksIcon *parent, QGraphicsSceneDragDropEvent *event);
        void addItem(QObject *object);
        int itemsAmount() const;
        bool isExecutable() const;
        KMenu* contextMenu();
        KIcon icon();
        KUrl url() const;
        QPixmap thumbnail();
        QString exec() const;
        QString title() const;
        QString description() const;

    public slots:
        void setUrl(KUrl url);
        void setThumbnail(const KFileItem item, const QPixmap thumbnail);
        void setBrowseMenu();
        void openUrl(QAction *action);
        void activate();
        void removeItem(QObject *object);
        void showPropertiesDialog();

    private:
        FancyTasksApplet *m_applet;
        KMimeType::Ptr m_mimeType;
        KUrl m_url;
        KUrl m_realUrl;
        KIcon m_icon;
        QPixmap m_thumbnail;
        QString m_exec;
        QString m_title;
        QString m_description;
        QList<QObject*> m_items;
        bool m_executable;

    signals:
        void changed();
        void hide();
        void show();
};

#endif
