/***********************************************************************************
* Fancy Tasks: Plasmoid for fancy representing your tasks and launchers.
* Copyright (C) 2009 Michal Dutkiewicz aka Emdek <emdeck@gmail.com>
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*
***********************************************************************************/

#include "FancyTasksSeparator.h"
#include "FancyTasksApplet.h"

FancyTasksSeparator::FancyTasksSeparator(Plasma::Svg *svg, FancyTasksApplet *applet) : Plasma::SvgWidget(svg, "separator", applet)
{
    setAcceptsHoverEvents(true);

    setSizePolicy(QSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed));

    m_applet = applet;

    m_visible = true;

    setObjectName("FancyTasksSeparator");

    connect(this, SIGNAL(hoverMoved(QObject*, qreal)), m_applet, SLOT(itemHoverMoved(QObject*, qreal)));
    connect(this, SIGNAL(hoverLeft()), m_applet, SLOT(itemHoverLeft()));
    connect(m_applet, SIGNAL(sizeChanged(qreal)), this, SLOT(setSize(qreal)));
}

void FancyTasksSeparator::hoverMoveEvent(QGraphicsSceneHoverEvent *event)
{
    emit hoverMoved(this, (qreal) (((m_applet->location() == Plasma::LeftEdge || m_applet->location() == Plasma::RightEdge)?event->pos().y():event->pos().x()) / m_size));
}

void FancyTasksSeparator::hoverLeaveEvent(QGraphicsSceneHoverEvent *event)
{
    Q_UNUSED(event)

    emit hoverLeft();
}

void FancyTasksSeparator::setSize(qreal size)
{
    m_size = size;

    if (m_visible)
    {
        if (m_applet->formFactor() == Plasma::Vertical)
        {
            setPreferredSize(m_size, (m_size / 4));
        }
        else
        {
            setPreferredSize((m_size / 4), m_size);
        }
    }
}

void FancyTasksSeparator::show()
{
    m_visible = true;

    setSize(m_size);

    m_applet->updateSize();
}

void FancyTasksSeparator::hide()
{
    m_visible = false;

    setPreferredSize(0, 0);

    m_applet->updateSize();
}

bool FancyTasksSeparator::isVisible()
{
    return m_visible;
}
